#!/usr/bin/perl
# Copyright (C) 2017 Yuri V. Kravatsky, jiri@eimb.ru - All Rights Reserved
# You may use, distribute and modify this code under the
# terms of the Creative Commons CC BY-NC-SA 3.0 license.
# If you use this script in your scienfic projects please cite:
# Kravatsky YV, Chechetkin VR, Fedoseeva DM, Gorbacheva MA, Kravatskaya GI, Kretova OV, Tchurikov NA.
# A bioinformatic pipeline for monitoring of the mutational stability of viral drug targets with deep-sequencing technology.
# Viruses 2017, 9, 357, DOI: 10.3390/v9120357
#
# Threshold is the amount of reads per sequence to save for future statistics. If you want to omit sequences that can be random
# mutations or sequencing reads you can set it to 2 (or any other reasonable value).
    my $THRESHOLD=1;

    use BioUtil::Seq;
    use strict;

    my $ARGC=scalar @ARGV;

    if ($ARGC!=2) { die "Usage: pack_fa.pl fastafile threshold\n"; }

    my $infile=$ARGV[0];
    my $basename=substr($infile,0,rindex($infile,"."));
    my $outfile=$basename.".packed.fa";
    if (! -f $infile) { print STDERR "Can't read from \"$infile\"!\n"; exit (-1); }
    open (OUTP,">$outfile") || die "Can't write to \"$outfile\": $!";

    $THRESHOLD=$ARGV[1];
    $_=$THRESHOLD;
    if ((! /^-?\d+\z/) || $THRESHOLD<=0 ) { die "Threshold must be integer value from 1 and above\n"; }

    my $next_seq = FastaReader($infile);

    my $k=0;
    my %fpack;
    while ( my $fa = &$next_seq() ) {
        my $header;
        my $seq;
        ($header, $seq) = @$fa;
        $fpack{$seq}++;
                                    }
     my $i=1;
     foreach my $key (sort { $fpack{$b} <=> $fpack{$a} } (keys(%fpack))) {
             if ($fpack{$key}<$THRESHOLD) { next; }
             print OUTP ">$i.$fpack{$key}\n$key\n";
             $i++;                                                       }
    close (OUTP);
