#!/usr/bin/perl
# Copyright (C) 2017 Yuri V. Kravatsky, jiri@eimb.ru - All Rights Reserved
# You may use, distribute and modify this code under the
# terms of the Creative Commons CC BY-NC-SA 3.0 license.
# If you use this script in your scienfic projects please cite:
# Kravatsky YV, Chechetkin VR, Fedoseeva DM, Gorbacheva MA, Kravatskaya GI, Kretova OV, Tchurikov NA.
# A bioinformatic pipeline for monitoring of the mutational stability of viral drug targets with deep-sequencing technology.
# DOI: XX.XXXX/XXXX, PMID: XXXXXX
#
# Threshold is the amount of reads per sequence to save for future statistics. If you want to omit sequences that can be random
# mutations or sequencing reads you can set it to 2 (or any other reasonable value).
    my $THRESHOLD=1;

    use Scalar::Util qw(looks_like_number);
    use BioUtil::Seq;
    use strict;

    my $ARGC=scalar @ARGV;

    if ($ARGC!=1) { die "Usage: conspack fastafile\n"; }

    my $infile=$ARGV[0];
    my $basename=substr($infile,0,rindex($infile,"."));
    my $outfile=$basename.".cons.fa";
    open (INP,"$infile") || die "Can't read from \"$infile\": $!";
    close (INP);
    open (OUTP,">$outfile") || die "Can't write to \"$outfile\": $!";

    my $next_seq = FastaReader($infile);

    my $k=0;
    my @conshash;
    my @seq;
    my @header;
    while ( my $fa = &$next_seq() ) {        
        ( $header[$k], $seq[$k] ) = @$fa;
        my @head=split(/\./,$header[$k]);
        my $numstrings=$head[1];
        if (!looks_like_number($numstrings)) { next; }
        if ($numstrings<$THRESHOLD) { next; }
        my $localseq=uc($seq[$k]);
        my @arr=split('',$localseq);
        my $arrlength = @arr;
        if ($k==0) { for (my $i=0;$i<$arrlength;$i++) {
                            $conshash[$i] {'A'} = 0;
                            $conshash[$i] {'C'} = 0;
                            $conshash[$i] {'G'} = 0;
                            $conshash[$i] {'T'} = 0;
                            $conshash[$i] {'N'} = 0;
                            $conshash[$i] {'-'} = 0;      }
                       }
        for (my $i=0;$i<$arrlength;$i++) { $conshash[$i] { $arr[$i] } += $numstrings; }
        $k++;
                                    }
     my $totalrecords=$k;

print "From $infile $totalrecords records read\n";
    my @consarray;
    my @conshort;
    for my $href ( @conshash ) {
       my $max_key;
       my $max_val=0;
       for my $role ( keys %$href ) {
            my $val=$href->{$role};
            ($max_key, $max_val) = ($role, $val) if $val > $max_val;
                                 }
       push @consarray, $max_key;
       if ($max_key ne "-") { push @conshort, $max_key; }
                            }
    my $constring=join('',@consarray);
    my $conshortstr=join('',@conshort);

    for (my $i=0;$i<$totalrecords;$i++) {
         my @shortarr;
         my @arr=split('',$seq[$i]);
         for (my $j=0;$j<scalar @arr;$j++) {
             if ($consarray[$j] ne "-") { push @shortarr, $arr[$j]; }
                                           }
         my $shortstr=join('',@shortarr);
         $shortstr=uc($shortstr);
         print OUTP ">$header[$i]\n$shortstr\n";
                                        }
#    print OUTP ">consensus\n$conshortstr\n$constring\n";
    print OUTP ">consensus\n$conshortstr\n";
    close (OUTP);
